/***************************************************************************
 *                                                                         *
 *   kemboss - Graphical user interface for EMBOSS                         *
 *                                                                         *
 *   Copyright (C) 2007-2019 Tobias Schwarz (mail@tobias-schwarz.net)      *
 *                                                                         *
 *   Based on the EMBOSS.kaptn scripts by Thomas Siegmund                  *
 *                                                                         *
 *   This program is free software; you can redistribute and/or modify     *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version, and provided that the above       *
 *   copyright and permission notice is included with all distributed      *
 *   copies of this or derived software.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 **************************************************************************/

#include <QApplication>
#include <QCommandLineParser>
#include <QMessageBox>
#include "main.h"
#include "kemboss.h"

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);
   QCoreApplication::setApplicationName("kemboss");
   QCoreApplication::setApplicationVersion(QString(KEMBOSS_MAJOR_VERSION)+"."+QString(KEMBOSS_MINOR_VERSION)+"."+QString(KEMBOSS_MINIMUM_VERSION));

   QCommandLineParser parser;
   parser.setSingleDashWordOptionMode(QCommandLineParser::ParseAsLongOptions);
   parser.setApplicationDescription("Graphical user interface for the sequence analysis package EMBOSS");
   parser.addHelpOption();
   parser.addVersionOption();

   // Start kemboss as portable app (-p, --portable)
   QCommandLineOption portableOption(QStringList() << "p" << "portable", "Run kemboss as a portable app. Configuration parameters are stored in config files.");
   parser.addOption(portableOption);

   // Process the actual command line arguments given by the user
   parser.process(app);

   kemboss kembossForm(nullptr, parser.isSet(portableOption));

   kembossForm.show();
   return app.exec();
}
