/***************************************************************************
 *                                                                         *
 *   kemboss - Graphical user interface for EMBOSS                         *
 *                                                                         *
 *   Copyright (C) 2007-2013 Tobias Schwarz (mail@tobias-schwarz.net)      *
 *                                                                         *
 *   Based on the EMBOSS.kaptn scripts by Thomas Siegmund                  *
 *                                                                         *
 *   This program is free software; you can redistribute and/or modify     *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version, and provided that the above       *
 *   copyright and permission notice is included with all distributed      *
 *   copies of this or derived software.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 **************************************************************************/

#include <QMessageBox>
#include "main.h"
#include "runEmbossAppLocal.h"
#include "programWindowClass.h"

krunEmbossAppLocal::krunEmbossAppLocal(QWidget *parent, kembossSettings *fksettings) : krunEmbossApp(parent, fksettings)
{
    // Standard output as well as standard error data are both shown in the programProgress window
    programProcess.setProcessChannelMode(QProcess::MergedChannels);

    // The program progress window is updated whenever new data is available
    connect(&programProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(updateProgramProgressForm()));
    connect(&programProcess, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(processFinished(int)));
}

bool krunEmbossAppLocal::checkEmbossApp()
{
   if (embossApp() == "chaos") // To avoid conflicting binary names, on some systems chaos is renamed into em_chaos
   {
      if (QFile::exists(ksettings->embossBin()+"em_chaos"))
         {
            setEmbossApp("em_chaos");
         }
   }

   // Check wether EMBOSS application exists
   if (!QFile::exists(ksettings->embossBin()+embossApp()
   #ifdef Q_OS_WIN32
   +".exe"
   #endif
   )) {
      QMessageBox::warning(parentWindow, objectName(),
                   tr("EMBOSS application ")+embossApp()+tr(" couldn't be found!\n"
                   "Please check Your kemboss settings."));
      return false;
   }
   return true;
}

bool krunEmbossAppLocal::runApp() {
    // On local EMBOSS installations, check if EMBOSS program binary exists
    if (!checkEmbossApp()) return false;
	
    // Do the basic preparations for running the EMBOSS app as defined in the krunEmbossApp base class
    krunEmbossApp::runApp();

    programProcess.start(ksettings->embossBin()+embossApp()
    #ifdef Q_OS_WIN32
    +".exe"
    #endif
    , programConfiguration);
    return true;
}

void krunEmbossAppLocal::updateProgramProgressForm()
{
    cprogramOutput.append(programProcess.readAll());
    krunEmbossApp::updateProgramProgressForm();
}

void krunEmbossAppLocal::processFinished(int exitCode)
{
    krunEmbossApp::processFinished(exitCode);
}

void krunEmbossAppLocal::killProcess()
{
    if (programProcess.state() != QProcess::NotRunning) programProcess.kill();
   
    krunEmbossApp::killProcess();
}
