/***************************************************************************
 *                                                                         *
 *   kemboss - Graphical user interface for EMBOSS                         *
 *                                                                         *
 *   Copyright (C) 2007-2013 Tobias Schwarz (mail@tobias-schwarz.net)      *
 *                                                                         *
 *   Based on the EMBOSS.kaptn scripts by Thomas Siegmund                  *
 *                                                                         *
 *   This program is free software; you can redistribute and/or modify     *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version, and provided that the above       *
 *   copyright and permission notice is included with all distributed      *
 *   copies of this or derived software.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 **************************************************************************/

#include <QSettings>
#include <QMessageBox>
#include <QCoreApplication>
#include <QProcess>
#include <QDir>

#include "settings.h"

kembossSettings::kembossSettings()
{

}

void kembossSettings::readSettings()
{
    // Load settings from configuration file / registry
    QSettings *settings;
    if (portableApp()) {
        settings = new QSettings(kembossPortableConfigDir()+"Configuration", QSettings::IniFormat);
    } else {
        settings = new QSettings(QSettings::UserScope, "kemboss.org", "Configuration");
    }
    ckembossData = settings->value("kemboss_data").toString();
    ckembossDefaultSettings = settings->value("kemboss_default_settings").toString();
    cembossBin = settings->value("emboss_bin").toString();
    cembossHelp = settings->value("emboss_help").toString();

    cembossTest = settings->value("EMBOSS_TEST","").toString();
    if (!cembossTest.isEmpty()) {
        QDir dir(QDir::fromNativeSeparators(cembossTest));
        if (!dir.exists())
            QMessageBox::warning(this, "kemboss", tr("The EMBOSS_TEST dir doesn't exist. Either create it or change the kemboss configuration file / registry entry."));
    }

    cuseOnlineDoc = settings->value("use_online_doc", false).toBool();
    conlineDocSource = settings->value("online_doc_source", "http://www.kemboss.org/emboss/").toString();
    cembossData = settings->value("emboss_data").toString();
    ccurrentDir = settings->value("current_dir", QDir::homePath()).toString();
    copenLastSettings = settings->value("open_last_settings", false).toBool();
    cautoCloseProgressWindow = settings->value("auto_close_progress_window", false).toBool();
    cautoShowOutputTab = settings->value("auto_show_output_tab", true).toBool();
    cautoDeleteExistingOutputFile = settings->value("auto_delete_existing_output_file", true).toBool();
    cmaxExeTime = settings->value("max_execution_time", 30000).toInt();
    settextEditFont(settings->value("text_edit_font_family", "courier").toString(),settings->value("text_edit_font_size", 10).toInt());
    setwrapLines(settings->value("text_edit_wrap_lines", false).toBool());
    cystein.setForeground(settings->value("cystein_color",QColor(Qt::darkGreen)).value<QColor>());
    apolar_aa.setForeground(settings->value("apolar_aa_color",QColor(Qt::darkYellow)).value<QColor>());
    polar_aa.setForeground(settings->value("polar_aa_color",QColor(Qt::darkGreen)).value<QColor>());
    acidic_aa.setForeground(settings->value("acidic_aa_color",QColor(Qt::red)).value<QColor>());
    basic_aa.setForeground(settings->value("basic_aa_color",QColor(Qt::blue)).value<QColor>());
    dibasic_aa.setForeground(settings->value("dibasic_aa_color",QColor(Qt::blue)).value<QColor>());
    a_nuc.setForeground(settings->value("a_nuc_color",QColor(Qt::green)).value<QColor>());
    t_nuc.setForeground(settings->value("t_nuc_color",QColor(Qt::red)).value<QColor>());
    g_nuc.setForeground(settings->value("g_nuc_color",QColor(Qt::black)).value<QColor>());
    c_nuc.setForeground(settings->value("c_nuc_color",QColor(Qt::blue)).value<QColor>());
    seq_name.setForeground(Qt::black);
    seq_name.setFontWeight(QFont::Bold);
    comment.setForeground(Qt::gray);
    comment.setFontItalic(true);
    cystein.setFontWeight(QFont::Bold);

    settings->beginGroup("RemoteEMBOSSInstallation");
       cremoteEmboss = settings->value("UseRemoteEMBOSSInstallation", false).toBool();
       cHostname = settings->value("Hostname", "tobias-schwarz.selfhost.bz").toString();
       cPort = settings->value("Port", 5757).toInt();
       cUsername = settings->value("UserName", "").toString();
       cUserGroup = settings->value("UserGroup","").toString();
       cPwHash = settings->value("Password", "").toByteArray();
       cEmail = settings->value("Email", "").toString();
    settings->endGroup();
}

void kembossSettings::writeSettings() const
{
    // Save settings to configuration file / registry
    QSettings *settings;
    if (portableApp()) {
        settings = new QSettings(kembossPortableConfigDir()+"Configuration", QSettings::IniFormat);
    } else {
        settings = new QSettings(QSettings::UserScope, "kemboss.org", "Configuration");
    }
    settings->setValue("kemboss_data", ckembossData);
    settings->setValue("kemboss_default_settings", ckembossDefaultSettings);
    settings->setValue("emboss_bin", cembossBin);
    settings->setValue("emboss_help", cembossHelp);
    settings->setValue("EMBOSS_TEST", cembossTest);
    settings->setValue("use_online_doc", cuseOnlineDoc);
    settings->setValue("online_doc_source", conlineDocSource);
    settings->setValue("emboss_data", cembossData);
    settings->setValue("current_dir", ccurrentDir);
    settings->setValue("open_last_settings", copenLastSettings);
    settings->setValue("auto_close_progress_window", cautoCloseProgressWindow);
    settings->setValue("auto_show_output_tab", cautoShowOutputTab);
    settings->setValue("auto_delete_existing_output_file", cautoDeleteExistingOutputFile);
    settings->setValue("max_execution_time", cmaxExeTime);
    settings->setValue("text_edit_font_family", textEditFont().family());
    settings->setValue("text_edit_font_size", textEditFont().pointSize());
    settings->setValue("text_edit_wrap_lines", wrapLines());
    settings->setValue("cystein_color",cystein.foreground().color());
    settings->setValue("apolar_aa_color",apolar_aa.foreground().color());
    settings->setValue("polar_aa_color",polar_aa.foreground().color());
    settings->setValue("acidic_aa_color",acidic_aa.foreground().color());
    settings->setValue("basic_aa_color",basic_aa.foreground().color());
    settings->setValue("dibasic_aa_color",dibasic_aa.foreground().color());
    settings->setValue("a_nuc_color",a_nuc.foreground().color());
    settings->setValue("t_nuc_color",t_nuc.foreground().color());
    settings->setValue("g_nuc_color",g_nuc.foreground().color());
    settings->setValue("c_nuc_color",c_nuc.foreground().color());

    settings->beginGroup("RemoteEMBOSSInstallation");
       settings->setValue("UseRemoteEMBOSSInstallation", cremoteEmboss);
       settings->setValue("Hostname", cHostname);
       settings->setValue("Port", cPort);
       settings->setValue("UserName", cUsername);
       settings->setValue("UserGroup", cUserGroup);
       settings->setValue("Password", cPwHash);
       settings->setValue("Email", cEmail);
    settings->endGroup();
}

bool kembossSettings::checkKembossDirectories()
{
    bool allDirsExist = true;
    if (ckembossData.isEmpty() || !QDir(ckembossData).exists()) {
        #ifdef Q_OS_WIN32
        if (QDir(QCoreApplication::applicationDirPath()+"\\data").exists()) setkembossData(QCoreApplication::applicationDirPath()+"\\data\\");
        else
        #else
            #ifdef Q_OS_MAC
            if (QDir(QCoreApplication::applicationDirPath()+"/../Resources/data").exists()) setkembossData(QCoreApplication::applicationDirPath()+"/../Resources/data/");
            else if (QDir("/Applications/kemboss.app/Contents/Resources/data").exists()) setkembossData("/Applications/kemboss.app/Contents/Resources/data/");
            else
            #endif
        if (QDir("/usr/share/kemboss/data").exists()) setkembossData("/usr/share/kemboss/data/");
        else if (QDir("/usr/local/share/kemboss/data").exists()) setkembossData("/usr/local/share/kemboss/data/");
        else if (QDir("/sw/share/kemboss/data").exists()) setkembossData("/sw/share/kemboss/data/");
        else
        #endif
        allDirsExist = false;
    }
    if (ckembossDefaultSettings.isEmpty() || !QDir(ckembossDefaultSettings).exists()) {
        #ifdef Q_OS_WIN32
        if (QDir(QCoreApplication::applicationDirPath()+"\\default_settings").exists()) setkembossDefaultSettings(QCoreApplication::applicationDirPath()+"\\default_settings\\");
        else
        #else
            #ifdef Q_OS_MAC
            if (QDir(QCoreApplication::applicationDirPath()+"/../Resources/default_settings").exists()) setkembossData(QCoreApplication::applicationDirPath()+"/../Resources/default_settings/");
            else if (QDir("/Applications/kemboss.app/Contents/Resources/default_settings").exists()) setkembossData("/Applications/kemboss.app/Contents/Resources/default_settings/");
            else
            #endif
        if (QDir("/usr/share/kemboss/default_settings").exists()) setkembossDefaultSettings("/usr/share/kemboss/default_settings/");
        else if (QDir("/usr/local/share/kemboss/default_settings").exists()) setkembossDefaultSettings("/usr/local/share/kemboss/default_settings/");
        else if (QDir("/etc/kemboss/default_settings").exists()) setkembossDefaultSettings("/etc/kemboss/default_settings/");
        else if (QDir("/sw/etc/kemboss/default_settings").exists()) setkembossDefaultSettings("/sw/etc/kemboss/default_settings/");
        else
        #endif
        allDirsExist = false;
    }
    if (!allDirsExist)
        QMessageBox::warning(this, "kemboss", tr("One or more kemboss directories are not specified. Please set the correct paths."));
    return allDirsExist;
}

bool kembossSettings::checkEmbossDirectories()
{
   bool allDirsExist = true;
   if (cembossBin.isEmpty() || !QDir(cembossBin).exists()) {
       #ifdef Q_OS_WIN32
       QStringList environment = QProcess::systemEnvironment();
       int i = environment.indexOf(QRegExp("^EMBOSS_ROOT=(.*)", Qt::CaseInsensitive));
       if (i >= 0 && QDir(environment.at(i).mid(12)).exists()) setembossBin(environment.at(i).mid(12));
       else
       #else
       if (QFile::exists("/usr/bin/embossversion")) setembossBin("/usr/bin/");
       else if (QFile::exists("/usr/local/bin/embossversion")) setembossBin("/usr/local/bin/");
       else if (QFile::exists("/sw/bin/embossversion")) setembossBin("/sw/bin/");
       else
       #endif
       allDirsExist = false;
   }
   if (!cuseOnlineDoc && (cembossHelp.isEmpty() || !QDir(cembossHelp).exists())) {
       #ifdef Q_OS_WIN32
       if (QDir(embossBin()+"doc\\programs\\html").exists()) setembossHelp(embossBin()+"doc\\programs\\html\\");
       else
       #else
       if (QDir("/usr/share/EMBOSS/doc/html/emboss/apps").exists()) setembossHelp("/usr/share/EMBOSS/doc/html/emboss/apps/");
       else if (QDir("/usr/share/EMBOSS/doc/programs/html").exists()) setembossHelp("/usr/share/EMBOSS/doc/programs/html/");
       else if (QDir("/usr/local/share/EMBOSS/doc/html/emboss/apps").exists()) setembossHelp("/usr/local/share/EMBOSS/doc/html/emboss/apps/");
       else if (QDir("/usr/local/share/EMBOSS/doc/programs/html").exists()) setembossHelp("/usr/local/share/EMBOSS/doc/programs/html/");
       else if (QDir("/usr/local/share/doc/packages/EMBOSS/html/emboss/apps").exists()) setembossHelp("/usr/local/share/doc/packages/EMBOSS/html/emboss/apps/");
       else if (QDir("/usr/local/share/doc/packages/EMBOSS/programs/html").exists()) setembossHelp("/usr/local/share/doc/packages/EMBOSS/programs/html/");
       else if (QDir("/sw/share/EMBOSS/doc/programs/html").exists()) setembossHelp("/sw/share/EMBOSS/doc/programs/html/");
       else
       #endif
       allDirsExist = false;
   }
   if (cembossData.isEmpty() || !QDir(cembossData).exists()) {
       #ifdef Q_OS_WIN32
       QStringList environment = QProcess::systemEnvironment();
       int i = environment.indexOf(QRegExp("^EMBOSS_DATA=(.*)", Qt::CaseInsensitive));
       if (i >= 0 && QDir(environment.at(i).mid(12)).exists()) setembossData(environment.at(i).mid(12));
       else if (QDir(embossBin()+"data").exists()) setembossData(embossBin()+"data\\");
       else
       #else
       if (QDir("/usr/share/EMBOSS/data").exists()) setembossData("/usr/share/EMBOSS/data/");
       else if (QDir("/usr/local/share/EMBOSS/data").exists()) setembossData("/usr/local/share/EMBOSS/data/");
       else if (QDir("/sw/share/EMBOSS/data").exists()) setembossData("/sw/share/EMBOSS/data/");
       else
       #endif
       allDirsExist = false;
   }
   if (cembossTest.isEmpty()) {
       #ifdef Q_OS_WIN32
       if (QDir(embossBin()+"test").exists()) setembossTest(embossBin()+"test");
       #else
       if (QDir("/usr/share/EMBOSS/test").exists()) setembossTest("/usr/share/EMBOSS/test");
       else if (QDir("/usr/local/share/EMBOSS/test").exists()) setembossTest("/usr/local/share/EMBOSS/test");
       else if (QDir("/sw/share/EMBOSS/test/").exists()) setembossTest("/sw/share/EMBOSS/test");
       #endif
   }
   if (!allDirsExist)
       QMessageBox::warning(this, "kemboss", tr("One or more EMBOSS directories are not specified. Please check Your local EMBOSS installation and set the correct paths."));
   return allDirsExist;
}

bool kembossSettings::checkConfiguration()
{
    return true;
}

bool kembossSettings::checkServer()
{
    if (cremoteEmboss) {
        if (cHostname == "") { QMessageBox::warning(this, "kemboss", tr("You want to use kemboss as a client to access a remote EMBOSS installation but no hostname is defined. Please check the kemboss client settings.")); return(false); }
        if (cPort == 0)  { QMessageBox::warning(this, "kemboss", tr("You want to use kemboss as a client to access a remote EMBOSS installation but no port is defined. Please check the kemboss client settings.")); return(false); }
        if (cUsername == "") { QMessageBox::warning(this, "kemboss", tr("You want to use kemboss as a client to access a remote EMBOSS installation but no username is defined. Please check the kemboss client settings.")); return(false); }
        if (cPwHash == "" ) { QMessageBox::warning(this, "kemboss", tr("You want to use kemboss as a client to access a remote EMBOSS installation but no password is set. Please check the kemboss client settings.")); return(false); }

        cuseOnlineDoc = true;
    }
    return true;
}
